#ifndef MESH_MANAGER_H
#define MESH_MANAGER_H

#include <vector>
#include <vbuffer.h>
#include "geometry.h"

struct RGInfo
{
	unsigned int vb_id;
	unsigned int ib_id;
	unsigned int start_vertex;
	unsigned int vertex_count;
	unsigned int start_index;
	unsigned int index_count;
	unsigned int primitive_count;
};


struct MeshInfo
{
	RGInfo  *rg_info_ptr;
	Geometry *geom;
	unsigned int mat_id;
	VertexFormat vf;

	bool operator<(const MeshInfo& info) const;
};


class MeshBuffer
{
protected:
	std::vector<MeshInfo *> mesh_list;

	VertexFormat buffer_vertex_format;

	unsigned int hw_vbuffer;
	unsigned int hw_ibuffer;

	unsigned int total_vertices;
	unsigned int total_indices;

	void Release ();

public:
	MeshBuffer (VertexFormat &vf);
	~MeshBuffer ();

	bool AddMesh (MeshInfo *info);
	bool Build ();
	bool Render (unsigned int idx);
};


class MeshManager
{
protected:
	std::vector<MeshBuffer *> mesh_buffers;
	std::vector<MeshInfo> mesh_info;

	void ClearMeshBuffers ();
public:
	MeshManager ();
	~MeshManager ();

	void Release ();

	bool AddGeometry (Geometry *geom, unsigned int mat_id, const VertexFormat &vf, RGInfo *rg_info);
	bool Build ();
	
	unsigned int GetMeshCount ();
};


#endif //MESH_MANAGER_H

