#include "objectgroup.h"



ObjectGroup::ObjectGroup()
{
}

ObjectGroup::~ObjectGroup()
{
	delete [] filename;
}

void ObjectGroup::ProcessGeometry (MeshChunk* mesh, const Matrix4x4 &xform)
{
	Mesh *added_mesh = new Mesh (); 

	added_mesh->geom = new Geometry ();
	added_mesh->material_id = 0;

	mesh_list.push_back (added_mesh);



	GeometryArray* mesh_array = mesh->GetArray(POSITION_ARRAY);
	if (mesh_array)
	{
		VertexArray *g_vertex = added_mesh->geom->CreateVertexArray ();
		void *dest_data = g_vertex->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_vertex->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(NORMAL_ARRAY);
	if (mesh_array)
	{
		NormalArray *g_normal = added_mesh->geom->CreateNormalArray ();
		void *dest_data = g_normal->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_normal->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(TANGENT_ARRAY);
	if (mesh_array)
	{
		TangentArray *g_tangent = added_mesh->geom->CreateTangentArray ();
		void *dest_data = g_tangent->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_tangent->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(COLOR_ARRAY);
	if (mesh_array)
	{
		ColorArray *g_color = added_mesh->geom->CreateColorArray ();
		void *dest_data = g_color->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_color->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(INDEX_ARRAY);
	if (mesh_array)
	{
		IndexArray *g_index = added_mesh->geom->CreateIndexArray ();
		void *dest_data = g_index->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_index->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(TEXCOORD0_ARRAY);
	if (mesh_array)
	{
		TexCoordArray *g_texcoord = added_mesh->geom->CreateTexCoordArray (0, mesh_array->GetBasicTypeCount());
		void *dest_data = g_texcoord->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_texcoord->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(TEXCOORD1_ARRAY);
	if (mesh_array)
	{
		TexCoordArray *g_texcoord = added_mesh->geom->CreateTexCoordArray (0, mesh_array->GetBasicTypeCount());
		void *dest_data = g_texcoord->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_texcoord->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(TEXCOORD2_ARRAY);
	if (mesh_array)
	{
		TexCoordArray *g_texcoord = added_mesh->geom->CreateTexCoordArray (0, mesh_array->GetBasicTypeCount());
		void *dest_data = g_texcoord->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_texcoord->GetSingleElementSize());
	}

	mesh_array = mesh->GetArray(TEXCOORD3_ARRAY);
	if (mesh_array)
	{
		TexCoordArray *g_texcoord = added_mesh->geom->CreateTexCoordArray (0, mesh_array->GetBasicTypeCount());
		void *dest_data = g_texcoord->Create (mesh_array->GetElementCount());
		memcpy (dest_data, mesh_array->GetArrayData(), mesh_array->GetElementCount()*g_texcoord->GetSingleElementSize());
	}

	added_mesh->geom->Transform (xform);
}


void ObjectGroup::ProcessNode (NodeChunk *node, const Matrix4x4 &xform)
{
	//printf("Node %d: %s\n", node->GetNodeID(), node->GetName().c_str());

	unsigned int node_meshes = node->GetMeshCount();
	for (unsigned int i = 0; i < node_meshes; i++)
	{
		ProcessGeometry(node->GetMesh(i), node->GetLocalPRS().GetTransform());
	}

	unsigned int node_proxys = node->GetProxyCount();

	unsigned int node_lights = node->GetLightCount();

	for (unsigned int i = 0; i < node_lights; i++)
	{
		//ProcessLight(node->GetLight(i));
	}

	// process cameras
	unsigned int node_cameras = node->GetCameraCount();
	for (unsigned int i = 0; i < node_cameras; i++)
	{
		//ProcessCamera(node->GetCamera(i));
	}
	for (unsigned int i = 0; i < node->GetSubNodeCount(); i++)
	{
		Matrix4x4 m;
		ProcessNode(node->GetSubNode(i), m);
	}
}


bool ObjectGroup::Load (const char *filename)
{
	VFS_FILE *fp = NULL;
	SceneFile *sce_file = NULL;

	try
	{
		fp = vfs_open (filename, "rb");
		if (fp == NULL)
		{
			throw (int) 0;
		}
		sce_file = new SceneFile ();

		if (sce_file->Load(fp) == false)
		{
			throw (int) 1;
		}
		vfs_close (fp);

		unsigned int root_node_count;
		root_node_count = sce_file->GetRootNodeCount ();

		for (unsigned int i = 0; i < root_node_count; i++)
		{
			Matrix4x4 m;
			ProcessNode(sce_file->GetRootNode(i), m);
		}

	}
	catch (...)
	{
		if (sce_file)
		{
			delete sce_file;
			sce_file = NULL;
		}
		if (fp)
		{
			vfs_close (fp);
		}
		return false;
	}

	this->filename = new char[strlen(filename) + 1];
	strcpy(this->filename, filename);

	return true;
}

const char ObjectGroup::GetFilename() const {
	return filename;
}

unsigned int ObjectGroup::GetMaterialCount () const
{
	return 0;
}

unsigned int ObjectGroup::GetMeshCount () const
{
	return (unsigned int) mesh_list.size ();
}

Mesh *ObjectGroup::GetMesh (unsigned int idx) const
{
	if (idx >= mesh_list.size ())
	{
		return 0;
	}
	return mesh_list[idx];
}

bool ObjectGroup::IsStatic () const
{
	return true;
}
