#ifndef OBJECT_GROUP_H
#define OBJECT_GROUP_H

#include <vector>
#include <libvfs.h>
#include <sce.h>
#include <anm.h>
#include "geometry.h"
#include "meshman.h"

struct Mesh
{
	Geometry *geom;
	unsigned int material_id;
	RGInfo rg_info;
};


class ObjectGroup 
{
protected:
	char *filename;	// needs to be kept because <level> just stores filenames
	std::vector<Mesh*> mesh_list;

	void ProcessGeometry (MeshChunk* mesh, const Matrix4x4 &xform);
	void ProcessNode (NodeChunk *node, const Matrix4x4 &xform);

public:
	ObjectGroup();
	~ObjectGroup();

	bool Load (const char *filename);
	const char *GetFilename() const;

	unsigned int GetMaterialCount () const;
	unsigned int GetMeshCount () const;
	Mesh *GetMesh (unsigned int idx) const;

	bool IsStatic () const;
};


#endif //OBJECT_GROUP_H

